
//----------
// transform functions
//----------

internal mat4
genCameraTransform( vec3 position, vec3 xAxis, vec3 yAxis, vec3 zAxis ) {
	mat3 rotMatrix   = mat3_byRow( xAxis, yAxis, zAxis );
	vec3 adjPosition = -( rotMatrix * position );
	
	mat4 result = {};
	result.elem[  0 ] = rotMatrix.elem[ 0 ];
	result.elem[  1 ] = rotMatrix.elem[ 1 ];
	result.elem[  2 ] = rotMatrix.elem[ 2 ];
	result.elem[  3 ] = 0.0f;
	result.elem[  4 ] = rotMatrix.elem[ 3 ];
	result.elem[  5 ] = rotMatrix.elem[ 4 ];
	result.elem[  6 ] = rotMatrix.elem[ 5 ];
	result.elem[  7 ] = 0.0f;
	result.elem[  8 ] = rotMatrix.elem[ 6 ];
	result.elem[  9 ] = rotMatrix.elem[ 7 ];
	result.elem[ 10 ] = rotMatrix.elem[ 8 ];
	result.elem[ 11 ] = 0.0f;
	result.elem[ 12 ] = adjPosition.x;
	result.elem[ 13 ] = adjPosition.y;
	result.elem[ 14 ] = adjPosition.z;
	result.elem[ 15 ] = 1.0f;
	return result;
}

internal mat4
genModelTransform( vec3 position, vec3 scale, quat orientation ) {
	mat3 rot = mat3_toRotation( orientation );
	
	mat4 result = {};
	result.elem[  0 ] = rot.elem[ 0 ] * scale.x;
	result.elem[  1 ] = rot.elem[ 1 ] * scale.x; 
	result.elem[  2 ] = rot.elem[ 2 ] * scale.x;
	result.elem[  4 ] = rot.elem[ 3 ] * scale.y;
	result.elem[  5 ] = rot.elem[ 4 ] * scale.y;
	result.elem[  6 ] = rot.elem[ 5 ] * scale.y;
	result.elem[  8 ] = rot.elem[ 6 ] * scale.z;
	result.elem[  9 ] = rot.elem[ 7 ] * scale.z;
	result.elem[ 10 ] = rot.elem[ 8 ] * scale.z;
	result.elem[ 12 ] = position.x;
	result.elem[ 13 ] = position.y;
	result.elem[ 14 ] = position.z;
	result.elem[ 15 ] = 1.0f;
	return result;
}

internal void
updateBallCollision( APP_STATE * appState, ASSET_SYSTEM * assetSystem, flo32 dt ) {
	PROFILE_FUNCTION();
	
	flo32 court_halfWidth  = COURT__HALF_WIDTH;
	flo32 court_halfLength = COURT__HALF_LENGTH;
	flo32 ball_radius      = BALL__RADIUS;
	
	vec3 hoopA_position = Vec3( 0.0f, 0.0f, -HOOP__DIST_FROM_CENTER );
	vec3 hoopB_position = Vec3( 0.0f, 0.0f,  HOOP__DIST_FROM_CENTER );
	
	vec3 accel = Vec3( 0.0f, -1.0f, 0.0f ) * 10.0f;
	vec3 dVel = accel * dt;
	vec3 dPos = accel * ( 0.5f * dt * dt ) + appState->ball_velocity * dt;
	appState->ball_velocity += dVel;
	
	vec3 collider_pos  = appState->ball_position;
	vec3 collider_dPos = dPos;
	
	flo32 tMin = 1.0f;
	flo32 tMinLeft = 1.0f;
	
	flo32 tMin_reflect    = 0.0f;
	vec3  tMin_normal     = {};
	vec3  tMin_prevNormal = {};
	boo32 killVelocity = false;
	
	COLLISION_3D collision = {};
	
	int32 normal_count = 0;
	vec3   normal[ 4 ]  = {};
	flo32 reflect[ 4 ] = {};
	if( getLengthSq( collider_dPos ) > COLLISION__VELOCITY_EPSILON ) {
		for( int32 collision_iter = 0; ( tMinLeft > COLLISION__COLLISION_EPSILON ) && ( collision_iter < 4 ); collision_iter++ ) {		
			if( getLengthSq( collider_dPos ) > COLLISION__VELOCITY_EPSILON ) {
				vec3 court_pos[] = {
					Vec3( -court_halfWidth, 0.0f,  court_halfLength ), Vec3(  court_halfWidth, 0.0f,  court_halfLength ),
					Vec3(  court_halfWidth, 0.0f, -court_halfLength ), Vec3( -court_halfWidth, 0.0f, -court_halfLength ),
				};
				
				collision = collision_quad( collider_pos, collider_dPos, court_pos, ball_radius );
				if( ( collision.isValid ) && ( collision.t < tMin ) ) {
					tMin         = collision.t;
					tMin_normal  = collision.normal;
					tMin_reflect = 0.85f;
				}
				
				COLLISION_GEOMETRY * stadiumCollision = getCollision( assetSystem, assetID_collision_stadiumFloor );
				collision = collision_sphere( collider_pos, collider_dPos, ball_radius, stadiumCollision );
				if( ( collision.isValid ) && ( collision.t < tMin ) ) {
					tMin         = collision.t;
					tMin_normal  = collision.normal;
					tMin_reflect = 0.85f;
				}
				
				COLLISION_GEOMETRY * hoopCollision = getCollision( assetSystem, assetID_collision_basketballHoop );
				collision = collision_sphere( collider_pos - hoopA_position, collider_dPos, ball_radius, hoopCollision );
				if( ( collision.isValid ) && ( collision.t < tMin ) ) {
					tMin         = collision.t;
					tMin_normal  = collision.normal;
					tMin_reflect = 0.85f;
				}
				
				vec3 vector0 = collider_pos - hoopB_position;
				vec3    pos0 = Vec3( -vector0.x, vector0.y, -vector0.z );
				vec3   dPos0 = Vec3( -collider_dPos.x, collider_dPos.y, -collider_dPos.z );
				collision = collision_sphere( pos0, dPos0, ball_radius, hoopCollision );
				if( ( collision.isValid ) && ( collision.t < tMin ) ) {
					tMin         = collision.t;
					tMin_normal  = Vec3( -collision.normal.x, collision.normal.y, -collision.normal.z );
					tMin_reflect = 0.85f;
				}
				
				COLLISION_GEOMETRY * jumbotronCollision = getCollision( assetSystem, assetID_collision_jumbotron );
				collision = collision_sphere( collider_pos, collider_dPos, ball_radius, jumbotronCollision );
				if( ( collision.isValid ) && ( collision.t < tMin ) ) {
					tMin         = collision.t;
					tMin_normal  = collision.normal;
					tMin_reflect = 0.85f;
				}
				
				collider_pos += collider_dPos * tMin;
				if( tMin < tMinLeft ) {
					appState->ball_contactPos[    1 ] = appState->ball_contactPos[    0 ] - appState->ball_contactNormal[ 0 ] * ( ball_radius * 0.005f );
					appState->ball_contactNormal[ 1 ] = appState->ball_contactNormal[ 0 ];
					
					appState->ball_contactPos[    0 ] = collider_pos - tMin_normal * ( ball_radius * 0.99f );
					appState->ball_contactNormal[ 0 ] = tMin_normal;
					
					if( dot( tMin_prevNormal, tMin_normal ) < 0.0f ) {
						collider_dPos = Vec3( 0.0f, 0.0f, 0.0f );
						killVelocity  = true;
					} else {
						collider_dPos -= ( dot( collider_dPos, tMin_normal ) * tMin_normal );
						 normal[ normal_count ] = tMin_normal;
						reflect[ normal_count ] = tMin_reflect;
						normal_count++;
					}
					tMin_prevNormal = tMin_normal;
				}
				
				tMinLeft -= tMin;
				tMin = tMinLeft;
			} else {
				killVelocity = true;
			}
		}
	} else {
		killVelocity = true;
	}
	
	if( killVelocity ) {
		appState->ball_velocity = Vec3( 0.0f, 0.0f, 0.0f );
	} else {
		for( int32 normal_index = 0; normal_index < normal_count; normal_index++ ) {
			vec3   normal0 =  normal[ normal_index ];
			flo32 reflect0 = reflect[ normal_index ];
			vec3 adjVector = dot( appState->ball_velocity, normal0 ) * normal0;
			appState->ball_velocity -= ( adjVector + adjVector * reflect0 );
			
			appState->ball_orientation    = Quat( appState->ball_rotateAxis, appState->ball_radians ) * appState->ball_orientation;
			appState->ball_radians        = 0.0f;
			appState->ball_rotateAxis     = getNormal( cross( appState->ball_velocity, normal0 ) );
			
			vec3 directionAxis = getNormal( cross( normal0, appState->ball_rotateAxis ) );
			appState->ball_rotateVelocity = -( dot( appState->ball_velocity, directionAxis ) / ( ball_radius * 2.0f ) );
		}
	}
	appState->ball_position  = collider_pos;
	appState->ball_radians  += appState->ball_rotateVelocity * dt;
}

internal PERM_MESH
addPermMesh( RENDER_SYSTEM_3D * renderSystem, ASSET_ID meshID ) {
	RENDER_MESH mesh = getMesh( renderSystem->assetSystem, meshID );
	VERTEX_BUFFER * vertexBuffer = renderSystem->vertexBuffer;
	
	int32 vertex_index = getPermIndex( vertexBuffer );
	addPerm( vertexBuffer, mesh.vertex, mesh.vertex_count );
	
	PERM_MESH result = {};
	result.drawType     = mesh.drawType;
	result.vertex_index = vertex_index;
	result.vertex_count = mesh.vertex_count;
	
	return result;
}

internal void
genStadiumFan( APP_STATE * appState ) {
	int32 vertex_index = 0;
	int32   face_index = 0;
	
	flo32  shin_length = 0.5f;
	flo32 thigh_length = 0.5f;
	flo32 torso_length = 0.7f;
	flo32  head_length = 0.3f;
	flo32  hand_length = 0.1f;
	flo32   arm_length = 0.66f;
	
	flo32 lengthA = shin_length  + thigh_length;
	flo32 lengthB =      lengthA + torso_length;
	
	vec3 bone0 = Vec3( 0.0f, 0.5f,  0.0f );
	vec3 bone1 = Vec3( 0.0f, 1.0f,  0.0f );
	vec3 bone2 = Vec3( 0.0f, 1.59f, 0.0f );
	
	int32 position_count = 0;
	int32 posIndex_count = 0;
	int32 texCoord_count = 0;
	int32 texIndex_count = 0;
	
	vec3   * position = appState->fanMesh_position;
	vec2   * texCoord = appState->fanMesh_texCoord;
	uint32 * posIndex = appState->fanMesh_posIndex;
	uint32 * texIndex = appState->fanMesh_texIndex;
	
	int32 base_texCoord = texCoord_count;
	for( int32 iter = 0; iter < 8; iter++ ) {
		flo32 left  = 0.125f * ( flo32 )( iter     );
		flo32 right = 0.125f * ( flo32 )( iter + 1 );
		texCoord[ texCoord_count++ ] = Vec2( left,  0.0f );
		texCoord[ texCoord_count++ ] = Vec2( right, 0.0f );
		texCoord[ texCoord_count++ ] = Vec2( left,  0.4f );
		texCoord[ texCoord_count++ ] = Vec2( right, 0.4f );
		texCoord[ texCoord_count++ ] = Vec2( left,  0.8f );
		texCoord[ texCoord_count++ ] = Vec2( right, 0.8f );
		texCoord[ texCoord_count++ ] = Vec2( left,  1.0f );
		texCoord[ texCoord_count++ ] = Vec2( right, 1.0f );
	}
	
	{ // legs
		flo32 offset   = 0.01f;
		
		flo32 heightA = 0.0f;
		flo32 heightB = shin_length;
		flo32 heightC = shin_length + thigh_length;
		
		vec2  dim    = Vec2( 0.17f, 0.225f );
		rect  boundA  = rectLCD( Vec2( -offset, 0.0f ), dim );
		rect  boundB  = rectRCD( Vec2(  offset, 0.0f ), dim );
		
		vec2 texCoord[] = { Vec2( 0.0f, 0.0f ), Vec2( 1.0f, 0.0f ), Vec2( 1.0f, 1.0f ), Vec2( 0.0f, 1.0f ) };
		
		{ // lower leg A
			int32 base_position = position_count;
			
			position[ position_count++ ] = Vec3( boundA.left,  heightA, boundA.top    );
			position[ position_count++ ] = Vec3( boundA.right, heightA, boundA.top    );
			position[ position_count++ ] = Vec3( boundA.right, heightA, boundA.bottom );
			position[ position_count++ ] = Vec3( boundA.left,  heightA, boundA.bottom );
			position[ position_count++ ] = Vec3( boundA.left,  heightB, boundA.top    );
			position[ position_count++ ] = Vec3( boundA.right, heightB, boundA.top    );
			position[ position_count++ ] = Vec3( boundA.right, heightB, boundA.bottom );
			position[ position_count++ ] = Vec3( boundA.left,  heightB, boundA.bottom );
			
			uint32 posIndex_array[] = { 0, 1, 5,  0, 5, 4,   1, 2, 6,  1, 6, 5,   2, 3, 7,  2, 7, 6,   3, 0, 4,  3, 4, 7,   3, 2, 1,  3, 1, 0,   4, 5, 6,  4, 6, 7, };
			uint32 texIndex_array[] = { 0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2, };
			for( int32 iter = 0; iter < 36; iter++ ) { posIndex[ posIndex_count++ ] = base_position + posIndex_array[ iter ]; }
			for( int32 iter = 0; iter < 36; iter++ ) { texIndex[ texIndex_count++ ] = base_texCoord + texIndex_array[ iter ]; }
		}
		
		{ // lower leg B
			int32 base_position = position_count;
			
			position[ position_count++ ] = Vec3( boundB.left,  heightA, boundB.top    );
			position[ position_count++ ] = Vec3( boundB.right, heightA, boundB.top    );
			position[ position_count++ ] = Vec3( boundB.right, heightA, boundB.bottom );
			position[ position_count++ ] = Vec3( boundB.left,  heightA, boundB.bottom );
			position[ position_count++ ] = Vec3( boundB.left,  heightB, boundB.top    );
			position[ position_count++ ] = Vec3( boundB.right, heightB, boundB.top    );
			position[ position_count++ ] = Vec3( boundB.right, heightB, boundB.bottom );
			position[ position_count++ ] = Vec3( boundB.left,  heightB, boundB.bottom );
			
			uint32 posIndex_array[] = { 0, 1, 5,  0, 5, 4,   1, 2, 6,  1, 6, 5,   2, 3, 7,  2, 7, 6,   3, 0, 4,  3, 4, 7,   3, 2, 1,  3, 1, 0,   4, 5, 6,  4, 6, 7, };
			uint32 texIndex_array[] = { 0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2, };
			for( int32 iter = 0; iter < 36; iter++ ) { posIndex[ posIndex_count++ ] = base_position + posIndex_array[ iter ]; }
			for( int32 iter = 0; iter < 36; iter++ ) { texIndex[ texIndex_count++ ] = base_texCoord + texIndex_array[ iter ]; }
		}
		
		{ // upper legA
			int32 base_position = position_count;
			
			position[ position_count++ ] = Vec3( boundA.left,  heightB, boundA.top    ) - bone0;
			position[ position_count++ ] = Vec3( boundA.right, heightB, boundA.top    ) - bone0;
			position[ position_count++ ] = Vec3( boundA.right, heightB, boundA.bottom ) - bone0;
			position[ position_count++ ] = Vec3( boundA.left,  heightB, boundA.bottom ) - bone0;
			position[ position_count++ ] = Vec3( boundA.left,  heightC, boundA.top    ) - bone0;
			position[ position_count++ ] = Vec3( boundA.right, heightC, boundA.top    ) - bone0;
			position[ position_count++ ] = Vec3( boundA.right, heightC, boundA.bottom ) - bone0;
			position[ position_count++ ] = Vec3( boundA.left,  heightC, boundA.bottom ) - bone0;
			
			uint32 posIndex_array[] = { 0, 1, 5,  0, 5, 4,   1, 2, 6,  1, 6, 5,   2, 3, 7,  2, 7, 6,   3, 0, 4,  3, 4, 7,   3, 2, 1,  3, 1, 0,   4, 5, 6,  4, 6, 7, };
			uint32 texIndex_array[] = { 0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2, };
			for( int32 iter = 0; iter < 36; iter++ ) { posIndex[ posIndex_count++ ] = base_position + posIndex_array[ iter ]; }
			for( int32 iter = 0; iter < 36; iter++ ) { texIndex[ texIndex_count++ ] = base_texCoord + texIndex_array[ iter ]; }
		}
		
		{ // upper legB
			int32 base_position = position_count;
			
			position[ position_count++ ] = Vec3( boundB.left,  heightB, boundB.top    ) - bone0;
			position[ position_count++ ] = Vec3( boundB.right, heightB, boundB.top    ) - bone0;
			position[ position_count++ ] = Vec3( boundB.right, heightB, boundB.bottom ) - bone0;
			position[ position_count++ ] = Vec3( boundB.left,  heightB, boundB.bottom ) - bone0;
			position[ position_count++ ] = Vec3( boundB.left,  heightC, boundB.top    ) - bone0;
			position[ position_count++ ] = Vec3( boundB.right, heightC, boundB.top    ) - bone0;
			position[ position_count++ ] = Vec3( boundB.right, heightC, boundB.bottom ) - bone0;
			position[ position_count++ ] = Vec3( boundB.left,  heightC, boundB.bottom ) - bone0;
			
			uint32 posIndex_array[] = { 0, 1, 5,  0, 5, 4,   1, 2, 6,  1, 6, 5,   2, 3, 7,  2, 7, 6,   3, 0, 4,  3, 4, 7,   3, 2, 1,  3, 1, 0,   4, 5, 6,  4, 6, 7, };
			uint32 texIndex_array[] = { 0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2,   0, 1, 3,  0, 3, 2, };
			for( int32 iter = 0; iter < 36; iter++ ) { posIndex[ posIndex_count++ ] = base_position + posIndex_array[ iter ]; }
			for( int32 iter = 0; iter < 36; iter++ ) { texIndex[ texIndex_count++ ] = base_texCoord + texIndex_array[ iter ]; }
		}
	}
	
	{ // body
		flo32 heightA = lengthA - 0.1f;
		flo32 heightB = lengthB;
		flo32 width = 0.40f;
		flo32 depth = 0.25f;
		vec2  dim   = Vec2( width, depth );
		
		rect bound = rectCD( Vec2( 0.0f, 0.0f ), dim );
		
		int32 base_position = position_count;
		
		position[ position_count++ ] = Vec3( bound.left,  heightA, bound.top    ) - bone1;
		position[ position_count++ ] = Vec3( bound.right, heightA, bound.top    ) - bone1;
		position[ position_count++ ] = Vec3( bound.right, heightA, bound.bottom ) - bone1;
		position[ position_count++ ] = Vec3( bound.left,  heightA, bound.bottom ) - bone1;
		position[ position_count++ ] = Vec3( bound.left,  heightB, bound.top    ) - bone1;
		position[ position_count++ ] = Vec3( bound.right, heightB, bound.top    ) - bone1;
		position[ position_count++ ] = Vec3( bound.right, heightB, bound.bottom ) - bone1;
		position[ position_count++ ] = Vec3( bound.left,  heightB, bound.bottom ) - bone1;
		
		uint32 posIndex_array[] = { 0, 1, 5,  0, 5, 4,   1, 2, 6,  1, 6, 5,   2, 3, 7,  2, 7, 6,   3, 0, 4,  3, 4, 7,   3, 2, 1,  3, 1, 0,   4, 5, 6,  4, 6, 7, };
		uint32 texIndex_array[] = { 2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4, };
		for( int32 iter = 0; iter < 36; iter++ ) { posIndex[ posIndex_count++ ] = base_position + posIndex_array[ iter ]; }
		for( int32 iter = 0; iter < 36; iter++ ) { texIndex[ texIndex_count++ ] = base_texCoord + texIndex_array[ iter ]; }
	}
	
	{ // head
		flo32 heightA = lengthB;
		flo32 heightB = lengthB + head_length;
		flo32 width = 0.25f;
		flo32 depth = 0.25f;
		vec2  dim   = Vec2( width, depth );
		
		rect bound = rectCD( Vec2( 0.0f, 0.0f ), dim );
		bound.top += 0.05f;
		
		int32 base_position = position_count;
		
		position[ position_count++ ] = Vec3( bound.left,  heightA, bound.top    ) - bone1;
		position[ position_count++ ] = Vec3( bound.right, heightA, bound.top    ) - bone1;
		position[ position_count++ ] = Vec3( bound.right, heightA, bound.bottom ) - bone1;
		position[ position_count++ ] = Vec3( bound.left,  heightA, bound.bottom ) - bone1;
		position[ position_count++ ] = Vec3( bound.left,  heightB, bound.top    ) - bone1;
		position[ position_count++ ] = Vec3( bound.right, heightB, bound.top    ) - bone1;
		position[ position_count++ ] = Vec3( bound.right, heightB, bound.bottom ) - bone1;
		position[ position_count++ ] = Vec3( bound.left,  heightB, bound.bottom ) - bone1;
		
		uint32 posIndex_array[] = { 0, 1, 5,  0, 5, 4,   1, 2, 6,  1, 6, 5,   2, 3, 7,  2, 7, 6,   3, 0, 4,  3, 4, 7,   3, 2, 1,  3, 1, 0,   4, 5, 6,  4, 6, 7, };
		uint32 texIndex_array[] = { 4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6, };
		for( int32 iter = 0; iter < 36; iter++ ) { posIndex[ posIndex_count++ ] = base_position + posIndex_array[ iter ]; }
		for( int32 iter = 0; iter < 36; iter++ ) { texIndex[ texIndex_count++ ] = base_texCoord + texIndex_array[ iter ]; }
	}
	
	{ // arms
		flo32 radians = ( PI * ( 10.0f / 180.0f ) );
		
		flo32 width  = 0.1f;
		flo32 depth  = 0.2f;
		
		flo32 halfDepth = depth * 0.5f;
		
		flo32 xPos[] = { -0.2f - width, -0.2f, 0.2f, 0.2f + width };
		flo32 yPos[] = { lengthB - arm_length - 0.01f, lengthB - arm_length - 0.01f + hand_length, lengthB - 0.01f };
		flo32 zPos[] = { halfDepth, -halfDepth };
		
		{ // arm a
			int32 base_position = position_count;
			
			position[ position_count++ ] = Vec3( xPos[ 0 ], yPos[ 0 ], zPos[ 0 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 1 ], yPos[ 0 ], zPos[ 0 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 1 ], yPos[ 0 ], zPos[ 1 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 0 ], yPos[ 0 ], zPos[ 1 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 0 ], yPos[ 1 ], zPos[ 0 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 1 ], yPos[ 1 ], zPos[ 0 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 1 ], yPos[ 1 ], zPos[ 1 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 0 ], yPos[ 1 ], zPos[ 1 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 0 ], yPos[ 2 ], zPos[ 0 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 1 ], yPos[ 2 ], zPos[ 0 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 1 ], yPos[ 2 ], zPos[ 1 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 0 ], yPos[ 2 ], zPos[ 1 ] ) - bone2;
			
			uint32 posIndex_array[] = { 0, 1, 5,  0, 5, 4,   1, 2,  6,  1,  6, 5,   2, 3,  7,  2,  7,  6,   3, 0, 4,  3, 4,  7,   3, 2,  1,  3,  1,  0,
			                            4, 5, 9,  4, 9, 8,   5, 6, 10,  5, 10, 9,   6, 7, 11,  6, 11, 10,   7, 4, 8,  7, 8, 11,   8, 9, 10,  8, 10, 11, };
			uint32 texIndex_array[] = { 4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6,
			                            2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4, };
			for( int32 iter = 0; iter < 60; iter++ ) { posIndex[ posIndex_count++ ] = base_position + posIndex_array[ iter ]; }
			for( int32 iter = 0; iter < 60; iter++ ) { texIndex[ texIndex_count++ ] = base_texCoord + texIndex_array[ iter ]; }
		}
		
		{ // arm b
			int32 base_position = position_count;
			
			position[ position_count++ ] = Vec3( xPos[ 2 ], yPos[ 0 ], zPos[ 0 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 3 ], yPos[ 0 ], zPos[ 0 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 3 ], yPos[ 0 ], zPos[ 1 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 2 ], yPos[ 0 ], zPos[ 1 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 2 ], yPos[ 1 ], zPos[ 0 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 3 ], yPos[ 1 ], zPos[ 0 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 3 ], yPos[ 1 ], zPos[ 1 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 2 ], yPos[ 1 ], zPos[ 1 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 2 ], yPos[ 2 ], zPos[ 0 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 3 ], yPos[ 2 ], zPos[ 0 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 3 ], yPos[ 2 ], zPos[ 1 ] ) - bone2;
			position[ position_count++ ] = Vec3( xPos[ 2 ], yPos[ 2 ], zPos[ 1 ] ) - bone2;
			
			uint32 posIndex_array[] = { 0, 1, 5,  0, 5, 4,   1, 2,  6,  1,  6, 5,   2, 3,  7,  2,  7,  6,   3, 0, 4,  3, 4,  7,   3, 2,  1,  3,  1,  0,
			                            4, 5, 9,  4, 9, 8,   5, 6, 10,  5, 10, 9,   6, 7, 11,  6, 11, 10,   7, 4, 8,  7, 8, 11,   8, 9, 10,  8, 10, 11, };
			uint32 texIndex_array[] = { 4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6,   4, 5, 7,  4, 7, 6,
			                            2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4,   2, 3, 5,  2, 5, 4, };
			for( int32 iter = 0; iter < 60; iter++ ) { posIndex[ posIndex_count++ ] = base_position + posIndex_array[ iter ]; }
			for( int32 iter = 0; iter < 60; iter++ ) { texIndex[ texIndex_count++ ] = base_texCoord + texIndex_array[ iter ]; }
		}
	}
}

internal flo32
getYPos( ASSET_SYSTEM * assetSystem, vec2 originPos ) {
	PROFILE_FUNCTION();
	
	flo32 result = 0.0f;
	
	vec3 origin = Vec3( originPos.x, 4.0f, originPos.y );
	vec3 vector = Vec3( 0.0f, -10.0f, 0.0f );
	
	flo32 tMin         = 1000000.0f;
	boo32 tMin_isValid = false;
	
	COLLISION_GEOMETRY * stadiumGeometry = getCollision( assetSystem, assetID_collision_stadiumFloor );
	COLLISION_3D collision = collision_ray( origin, vector, stadiumGeometry );
	if( ( collision.isValid ) && ( collision.t < tMin ) ) {
		tMin_isValid = true;
		tMin         = collision.t;
	}
	
	if( tMin_isValid ) {
		result = origin.y + vector.y * tMin;
	}
	
	return result;
}

internal void
updatePlayerMovement( APP_STATE * appState, ASSET_SYSTEM * assetSystem, flo32 dt, flo32 radius ) {
	PROFILE_FUNCTION();
	
	vec2 player_hAxis = appState->player_hAxis;
	vec2 player_vAxis = appState->player_vAxis;
	
	vec2 accel = {};
	if( appState->move_left     ) { accel.x -= 1.0f; }
	if( appState->move_right    ) { accel.x += 1.0f; }
	if( appState->move_backward ) { accel.y -= 1.0f; }
	if( appState->move_forward  ) { accel.y += 1.0f; }
	accel = getNormal( accel );
	
	flo32 maxSpeed = 4.4704f; // 10 mph
	flo32 friction_constant = 8.0f;
	accel = player_hAxis * accel.x + player_vAxis * accel.y;
	accel = accel * maxSpeed * friction_constant - appState->player_velocity * friction_constant;
	
	vec2 dPos = accel * ( 0.5f * dt * dt ) + appState->player_velocity * dt;
	appState->player_velocity += ( accel * dt );
	
	vec2 collider_pos  = appState->player_position;
	vec2 collider_dPos = dPos;
	
	flo32 tMin     = 1.0f;
	flo32 tMinLeft = 1.0f;
	
	vec2  tMin_normal     = {};
	vec2  tMin_prevNormal = {};
	boo32 killVelocity = false;
	
	COLLISION_2D collision = {};
	
	int32 normal_count = 0;
	vec2   normal[ 4 ] = {};
	
	for( int32 collision_iter = 0; ( tMinLeft > COLLISION__TMIN_EPSILON ) && ( collision_iter < 4 ); collision_iter++ ) {		
		if( getLengthSq( collider_dPos ) > 0.0f ) {
			COLLISION_GEOMETRY * walkBoundGeometry = getCollision( assetSystem, assetID_collision_walkBound );
			collision = collision_circle( collider_pos, collider_dPos, radius, walkBoundGeometry );
			if( ( collision.isValid ) && ( collision.t < tMin ) ) {
				tMin         = collision.t;
				tMin_normal  = collision.normal;
			}
			
			collider_pos += collider_dPos * tMin;
			if( tMin < tMinLeft ) {
				if( dot( tMin_prevNormal, tMin_normal ) < -COLLISION__COLLISION_EPSILON ) {
					collider_dPos = Vec2( 0.0f, 0.0f );
					killVelocity  = true;
				} else {
					collider_dPos -= ( dot( collider_dPos, tMin_normal ) * tMin_normal );
					 normal[ normal_count ] = tMin_normal;
					normal_count++;
				}
				tMin_prevNormal = tMin_normal;
			}
			
			tMinLeft -= tMin;
			tMin = tMinLeft;
		} else {
			killVelocity = true;
		}
	}

	if( killVelocity ) {
		appState->player_velocity = Vec2( 0.0f, 0.0f );
	} else {
		for( int32 normal_index = 0; normal_index < normal_count; normal_index++ ) {
			vec2 normal0               = normal[ normal_index ];
			appState->player_velocity -= ( dot( appState->player_velocity, normal0 ) * normal0 );
		}
	}
	appState->player_position = collider_pos;
}

internal void
addVertex( MEMORY * memory, vec3 position, vec2 texCoord ) {
	_writem( memory, vec3, position );
	_writem( memory, vec2, texCoord );
}

internal void
addQuad( MEMORY * memory, vec3 posA, vec3 posB, vec3 posC, vec3 posD, vec2 * tex ) {
	addVertex( memory, posA, tex[ 0 ] );
	addVertex( memory, posB, tex[ 1 ] );
	addVertex( memory, posC, tex[ 2 ] );
	
	addVertex( memory, posA, tex[ 0 ] );
	addVertex( memory, posC, tex[ 2 ] );
	addVertex( memory, posD, tex[ 3 ] );
}

internal void
addQuad( MEMORY * memory, vec3 posA, vec3 posB, vec3 posC, vec3 posD, vec2 tex ) {
	addVertex( memory, posA, tex );
	addVertex( memory, posB, tex );
	addVertex( memory, posC, tex );
	
	addVertex( memory, posA, tex );
	addVertex( memory, posC, tex );
	addVertex( memory, posD, tex );
}

internal void
addTri( MEMORY * memory, vec3 posA, vec3 posB, vec3 posC, vec2 tex ) {
	addVertex( memory, posA, tex );
	addVertex( memory, posB, tex );
	addVertex( memory, posC, tex );
}

internal void
addTri( MEMORY * memory, vec3 posA, vec3 posB, vec3 posC, vec2 * tex ) {
	addVertex( memory, posA, tex[ 0 ] );
	addVertex( memory, posB, tex[ 1 ] );
	addVertex( memory, posC, tex[ 2 ] );
}

internal void
drawJumbotronText( RENDER_SYSTEM_3D * renderSystem, APP_STATE * appState ) {
	RENDER_MESH mesh             = getMesh(    renderSystem->assetSystem, assetID_mesh_jumbotronText  );
	uint32      texture_bufferID = getTexture( renderSystem->assetSystem, assetID_texture_jumbotronText ).bufferID;
	
	beginTask( renderSystem, meshDrawType_triangles, texture_bufferID, COLOR_WHITE );
	
	vec3 * vertex = ( vec3 * )mesh.vertex;
	vec3 basePos = vertex[ 0 ];
	vec3 xAxis0  = vertex[ 1 ];
	vec3 yAxis0  = vertex[ 2 ];
	vec3 xAxis1  = vertex[ 3 ];
	vec3 yAxis1  = vertex[ 4 ];
	
	int32 score_home = appState->scoreState_home.score;
	int32 score_away = appState->scoreState_away.score;
	
	int32 minutes = 0;
	int32 seconds = 0;
	if( appState->jumbotron_timer < 0.0f ) {
		minutes = 99;
		seconds = 99;
	} else {
		int32 time = ( int32 )appState->jumbotron_timer;
		minutes = time / 60;
		seconds = time % 60;
	}
	
	int32 tex_select[ 8 ] = {
		score_home / 10, score_home % 10, score_away / 10, score_away % 10,
		minutes    / 10,    minutes % 10,    seconds / 10,    seconds % 10,
	};
	
	flo32 xStride = 0.1f;
	vec2  tex[ 11 * 4 ] = {};
	int32 tex_index = 0;
	for( int32 iter = 0; iter < 10; iter++ ) {
		flo32 xPos0 = xStride * ( flo32 )( iter     );
		flo32 xPos1 = xStride * ( flo32 )( iter + 1 );
		
		tex[ tex_index++ ] = Vec2( xPos0, 0.0f );
		tex[ tex_index++ ] = Vec2( xPos1, 0.0f );
		tex[ tex_index++ ] = Vec2( xPos1, 1.0f );
		tex[ tex_index++ ] = Vec2( xPos0, 1.0f );
	}
	
	vec3 offset[] = { vertex[ 5 ], vertex[ 6 ], vertex[ 7 ], vertex[ 8 ], vertex[ 9 ], vertex[ 10 ], vertex[ 11 ], vertex[ 12 ], };
	for( int32 iter = 0; iter < 4; iter++ ) {
		for( int32 offset_index = 0; offset_index < 4; offset_index++ ) {
			vec3 basePos0 = basePos + offset[ offset_index ];
			vec3 pos[] = { basePos0, basePos0 + xAxis0, basePos0 + xAxis0 + yAxis0, basePos0 + yAxis0, };
			addQuad( renderSystem, pos, tex + ( tex_select[ offset_index ] * 4 ) );
		}
		for( int32 offset_index = 4; offset_index < 8; offset_index++ ) {
			vec3 basePos0 = basePos + offset[ offset_index ];
			vec3 pos[] = { basePos0, basePos0 + xAxis1, basePos0 + xAxis1 + yAxis1, basePos0 + yAxis1, };
			addQuad( renderSystem, pos, tex + ( tex_select[ offset_index ] * 4 ) );
		}
		basePos = Vec3( basePos.z, basePos.y, -basePos.x );
		xAxis0 = Vec3( xAxis0.z, xAxis0.y, -xAxis0.x );
		yAxis0 = Vec3( yAxis0.z, yAxis0.y, -yAxis0.x );
		xAxis1 = Vec3( xAxis1.z, xAxis1.y, -xAxis1.x );
		yAxis1 = Vec3( yAxis1.z, yAxis1.y, -yAxis1.x );
		for( int32 offset_index = 0; offset_index < 8; offset_index++ ) {
			vec3 offset0 = offset[ offset_index ];
			offset[ offset_index ] = Vec3( offset0.z, offset0.y, -offset0.x );
		}
	}
	endTask( renderSystem );
}

internal void
addColumnToWave( APP_STATE * appState, flo32 ratio ) {
	int32   select   = appState->fanWave_bufferSelect;
	int32 * buffer   = appState->fanWave_buffer + ( select * 20 );
	int32   inBuffer = appState->fanWave_inBuffer[ select ];
	
	int32 base  = appState->fanWave_column_baseIndex[  appState->fanWave_index ];
	int32 count = appState->fanWave_column_count[ appState->fanWave_index ];
	
	int32 flush_count = ( int32 )( ( flo32 )inBuffer * 0.5f );
	for( int32 flush_index = 0; flush_index < flush_count; flush_index++ ) {
		int32 select = rand() % inBuffer;
		if( appState->stadiumFan_radians[ buffer[ select ] ] > ( PI * 2.0f ) ) {
			appState->stadiumFan_radians[ buffer[ select ] ] = 0.0f;
			appState->stadiumFan_rate[     buffer[ select ] ] = 1.1f;
		}
		
		buffer[ select ] = buffer[ --inBuffer ];
	}
	
	for( int32 iter = 0; iter < count; iter++ ) {
		buffer[ iter ] = base + iter;
	}
	
	int32 remove_count = ( int32 )( ( flo32 )count * ( 1.0f - ratio ) );
	for( int32 remove_index = 0; remove_index < remove_count; remove_index++ ) {
		int32 index = rand() % count;
		buffer[ index ] = buffer[ --count ];
	}
	appState->fanWave_inBuffer[ select ] = count;
	
	appState->fanWave_index = ( appState->fanWave_index + 1 ) % 156;
}

internal void
initFanWave( APP_STATE * appState ) {
	appState->fanWave_timer = 0.0f;
	appState->fanWave_repeatCount = 80 + ( rand() % 110 ) + ( rand() % 110 );
	
	appState->fanWave_index = ( appState->fanWave_index + ( rand() % 100 ) ) % 156;
	
	for( int32 iter = 0; iter < 3; iter++ ) {
		addColumnToWave( appState, 0.4f );
		appState->fanWave_bufferSelect = ( appState->fanWave_bufferSelect + 1 ) % FAN_WAVE__BUFFER_COUNT;
	}
}

internal void
initFanWaveTimer( APP_STATE * appState ) {
	appState->fanWave_initTimer = 0.0f;
	appState->fanWave_initTargetTime = ( flo32 )( rand() % 40 ) + 20.0f;
}

internal void
updateFanWaveTimer( APP_STATE * appState, flo32 dt ) {
	if( appState->fanWave_repeatCount == 0 ) {
		appState->fanWave_initTimer += dt;
		if( appState->fanWave_initTimer >= appState->fanWave_initTargetTime ) {
			initFanWave( appState );
			initFanWaveTimer( appState );
		}
	}
}

internal void
updateFanWave( APP_STATE * appState, flo32 dt ) {
	flo32 fanBunch_targetTime = 0.825f * 0.225f;
	
	appState->fanWave_timer += dt;
	if( appState->fanWave_timer >= fanBunch_targetTime ) {
		appState->fanWave_timer = 0.0f;
		appState->fanWave_inBuffer[ appState->fanWave_bufferSelect ] = 0;
		if( appState->fanWave_repeatCount > 0 ) {
			flo32 ratio = minValue( ( flo32 )appState->fanWave_repeatCount * 0.05f, 1.0f );
			if( appState->fanWave_counter < 40 ) {
				ratio = lerp( 0.4f, ( flo32 )appState->fanWave_counter / 40.0f, 1.0f );
			}
			addColumnToWave( appState, ratio );
			appState->fanWave_repeatCount--;
			appState->fanWave_counter++;
		}
		appState->fanWave_bufferSelect = ( appState->fanWave_bufferSelect + 1 ) % FAN_WAVE__BUFFER_COUNT;
	}
	
	for( int32 select = 0; select < FAN_WAVE__BUFFER_COUNT; select++ ) {
		int32   inBuffer = appState->fanWave_inBuffer[ select ];
		
		if( ( inBuffer > 0 ) && ( ( rand() % 3 ) == 0 ) ) {
			int32 * buffer = appState->fanWave_buffer + ( select * 20 );
			int32   index  = rand() % inBuffer;
			if( appState->stadiumFan_radians[ buffer[ index ] ] > ( PI * 2.0f ) ) {
				appState->stadiumFan_radians[ buffer[ index ] ] = 0.0f;
				appState->stadiumFan_rate[     buffer[ index ] ] = 1.1f;
			}
			
			buffer[ index ] = buffer[ --appState->fanWave_inBuffer[ select ] ];
		}
	}
}

internal void
initFanCheer( APP_STATE * appState ) {
	appState->fanCheer_bufferCount  = 2248;
}

internal void
updateFanCheer( APP_STATE * appState, flo32 dRadians ) {
	int32 fan_count = minValue( appState->fanCheer_bufferCount, 75 );
	for( int32 iter = 0; iter < fan_count; iter++ ) {
		int32 select = rand() % appState->fanCheer_bufferCount;
		int32 fan_index = appState->fanCheer_buffer[ select ];
		
		int32 temp = appState->fanCheer_buffer[ appState->fanCheer_bufferCount - 1 ];
		appState->fanCheer_buffer[ appState->fanCheer_bufferCount - 1 ] = appState->fanCheer_buffer[ select ];
		appState->fanCheer_buffer[ select ] = temp;
		appState->fanCheer_bufferCount--;
		
		int32 repeatCount = ( rand() % 2 ) + 1;
		
		if( appState->stadiumFan_radians[ fan_index ] > ( PI * 2.0f ) ) {
			appState->stadiumFan_radians[         fan_index ] = 0.0f;
			appState->stadiumFan_rate[             fan_index ] = ( flo32 )( rand() % 41 ) / 100.0f + 0.8f;
			appState->stadiumFan_t[               fan_index ] = 1.0f;
			appState->fanCheer_repeatCount[ fan_index ] = repeatCount;
		} else {
			appState->fanCheer_repeatCount[ fan_index ] = repeatCount - 1;
		}
	}
	for( int32 fan_index = 0; fan_index < 2248; fan_index++ ) {
		flo32 rate = appState->stadiumFan_rate[ fan_index ];
		appState->stadiumFan_radians[ fan_index ] += ( dRadians * rate );
		if( ( appState->stadiumFan_radians[ fan_index ] > ( PI * 2.0f ) ) && ( appState->fanCheer_repeatCount[ fan_index ] > 0 ) ) {
			appState->fanCheer_repeatCount[ fan_index ]--;
			appState->stadiumFan_radians[         fan_index ] = 0.0f;
			appState->stadiumFan_rate[             fan_index ] = ( flo32 )( rand() % 41 ) / 100.0f + 0.8f;
		}
		appState->stadiumFan_t[       fan_index ]  = ( cosf( minValue( appState->stadiumFan_radians[ fan_index ], PI * 2.0f ) ) + 1.0f ) * 0.5f;
	}
}

#if DEBUG_BUILD
internal void
initTestShot( APP_STATE * appState, flo32 player_height, flo32 ball_radius ) {
	// vec2 player_position = Vec2( -26.0f, 16.0f );
	     // // player_position = Vec2(  -6.0f, 10.0f );
		  // player_position = Vec2( 0.0f, 0.0f );
	// vec2 shot_vector = Vec2( appState->hoop_position.x, appState->hoop_position.z + 0.1375f ) - player_position;
	
	// flo32 hRadians = atan2f( -shot_vector.y, shot_vector.x ) - ( PI * 0.5f );
	// flo32 hDegrees = hRadians / ( PI * 2.0f ) * 360.0f;
	// appState->shot_hDegrees = hDegrees;
	// flo32 vRadians = appState->player_shotPower * ( PI * 0.5f );
	      // vRadians = 31.5f / 360.0f * ( PI * 2.0f );
	
	// flo32 player_yPos   = getYPos( appState, player_position );
	// vec3  shot_position = Vec3( player_position.x, player_yPos + player_height, player_position.y );
	// flo32 shot_power    = appState->player_shotPower;
	
	// flo32 hSi = sinf( hRadians );
	// flo32 hCo = cosf( hRadians );
	
	// flo32 vSi = sinf( vRadians );
	// flo32 vCo = cosf( vRadians );
	
	// vec3 shot_xAxis = Vec3( hCo,       0.0f, -hSi       );
	// vec3 shot_yAxis = Vec3( hSi * vSi,  vCo,  hCo * vSi );
	// vec3 shot_zAxis = cross( shot_xAxis, shot_yAxis );
	
	appState->player_isHoldingBall = false;
	// appState->ball_position = shot_position - getNormal( shot_zAxis + shot_yAxis ) * ( 0.5f * 0.3048f + ball_radius );
	// appState->ball_velocity = getNormal( shot_position - shot_zAxis - appState->ball_position ) * ( shot_power * 17.575f + 2.0f );
	appState->ball_position = Vec3( 0.0f, 2.0f, 0.0f );
	appState->ball_velocity = Vec3( 0.0f, 0.0f, -2.0f );
	
	// vec3  tempVector  = getNormal( shot_position - shot_zAxis - appState->ball_position );
	// flo32 tempRadians = asinf( tempVector.y );
	// flo32 tempDegrees = tempRadians / ( PI * 2.0f ) * 360.0f;
	// CONSOLE_VALUE( flo32, tempDegrees );
	
	// appState->shot_position = shot_position;
	// appState->shot_ballPos  = appState->ball_position;
	// appState->shot_velocity = appState->ball_velocity;
	// appState->shot_xAxis    = shot_xAxis;
	// appState->shot_yAxis    = shot_yAxis;
	// appState->shot_zAxis    = shot_zAxis;
}
#endif // DEBUG_BUILD

internal void
updateScore( APP_STATE * appState, SCORE_STATE * scoreState, vec3 ball_position ) {
	flo32 rim_radius = RIM__MAJOR_RADIUS;
	cylinder upper = Cylinder( scoreState->position,                              Vec3( 0.0f, 0.25f, 0.0f ), rim_radius );
	cylinder lower = Cylinder( scoreState->position + Vec3( 0.0f, -0.25f, 0.0f ), Vec3( 0.0f, 0.25f, 0.0f ), rim_radius );

	vec4 upper_color = COLOR_BLUE;
	vec4 lower_color = COLOR_BLUE;
	
	if( scoreState->mode == scoreMode_isActive ) {
		lower_color = COLOR_GREEN;
	}
	if( scoreState->mode == scoreMode_isLocked ) {
		upper_color = COLOR_RED;
		lower_color = COLOR_RED;
	}
	debug_cylinder( upper, upper_color );
	debug_cylinder( lower, lower_color );
	
	boo32 isInUpperBound = isInCylinder( ball_position, upper );
	boo32 isInLowerBound = isInCylinder( ball_position, lower );
	
	if( scoreState->mode == scoreMode_isWaiting ) {
			  if( isInUpperBound ) { scoreState->mode = scoreMode_isActive; }
		else if( isInLowerBound ) { scoreState->mode = scoreMode_isLocked; }
	}
	if( scoreState->mode == scoreMode_isActive ) {
		if( isInLowerBound ) {
			scoreState->score++;
			scoreState->mode = scoreMode_isLocked;
			initFanCheer( appState );
		}
	}
	if( ( !isInLowerBound ) && ( ball_position.y < scoreState->position.y ) ) {
		scoreState->mode = scoreMode_isWaiting;
	}
}