
#ifdef 	STD_INCLUDE_DECL

struct RENDER_TEXTURE {
	uint32 bufferID;
	int32 width;
	int32 height;
};

struct RENDER_FONT {
	FONT   font;
	uint32 texture_bufferID;
};

struct RENDER_MONOSPACE_FONT {
	MONOSPACE_FONT	font;
	uint32	      texture_bufferID;
};

struct PERM_MESH {
	MESH_DRAW_TYPE drawType;
	int32 vertex_index;
	int32 vertex_count;
};

struct RENDER_MESH {
	MESH_DRAW_TYPE drawType;
	int32 vertex_count;
	VERTEX_TEXMESH_3D * vertex;
};

struct RENDER_OBJECT_2D {
	MESH_DRAW_TYPE drawType;
	
	uint32 vertex_index;
	uint32 vertex_count;
	
	uint32 texture_bufferID;
	vec4 color;
};

struct RENDER_OBJECT_3D {
	MESH_DRAW_TYPE drawType;
	
	uint32 vertex_index;
	uint32 vertex_count;
	
	vec3 position;
	vec3 scale;
	quat orientation;
	
	uint32 texture_bufferID;
	vec4 color;
};

struct VERTEX_LIST {
	int32 				  vertex_count;
	int32               vertex_maxCount;
	VERTEX_TEXMESH_3D * vertex;
};

struct VERTEX_BUFFER {
	VERTEX_TEXMESH_3D * buffer_base;
	
	int32 perm_baseIndex;
	int32 perm_maxCount;
	int32 perm_count;
	
	int32 temp_maxCount;
	int32 temp_count;
	int32 temp_select;
};

struct ASSET_SYSTEM;
#define RENDER_SYSTEM__MAX_VERTEX_COUNT	( 1048576 )
#define RENDER_SYSTEM__MAX_OBJECT_COUNT	( 32768 )

struct RENDER_SYSTEM_2D {
	ASSET_SYSTEM * assetSystem;
	
	boo32          taskIsActive;
	RENDER_OBJECT_2D  tempObject;
	
	int32 			   object_count;
	RENDER_OBJECT_2D 	object[ RENDER_SYSTEM__MAX_OBJECT_COUNT ];
	
	VERTEX_BUFFER * vertexBuffer;
};

struct RENDER_SYSTEM_3D {
	ASSET_SYSTEM * assetSystem;
	
	mat4 camera_transform;
	
	boo32          taskIsActive;
	RENDER_OBJECT_3D  tempObject;
	
	int32 			   object_count;
	RENDER_OBJECT_3D 	object[ RENDER_SYSTEM__MAX_OBJECT_COUNT ];
	
	VERTEX_BUFFER * vertexBuffer;
};

#endif 	// STD_INCLUDE_DECL
#ifdef	STD_INCLUDE_FUNC

//----------
// vertex list functions
//----------

internal VERTEX_LIST
VertexList( VERTEX_TEXMESH_3D * memory, int32 maxCount ) {
	VERTEX_LIST result = { 0, maxCount, memory };
	return result;
}

internal VERTEX_LIST
VertexList( MEMORY * memory, int32 maxCount ) {
	VERTEX_TEXMESH_3D * vertex = _pushArray_clear( memory, VERTEX_TEXMESH_3D, maxCount );
	VERTEX_LIST result = VertexList( vertex, maxCount );
	return result;
}

internal void
free( MEMORY * memory, VERTEX_LIST * vertexList ) {
	_popArray( memory, VERTEX_TEXMESH_3D, vertexList->vertex_maxCount );
}

internal void
addVertex( VERTEX_LIST * vertexList, VERTEX_TEXMESH_3D vertex ) {
	Assert( vertexList->vertex_count < vertexList->vertex_maxCount );
	vertexList->vertex[ vertexList->vertex_count++ ] = vertex;
}

internal void
addVertex( VERTEX_LIST * vertexList, vec3 position, vec2 texCoord ) {
	VERTEX_TEXMESH_3D vertex = VertexTexMesh3D( position, texCoord );
	addVertex( vertexList, vertex );
}

//----------
// render object functions
//----------

internal RENDER_OBJECT_2D
RenderObject2D( MESH_DRAW_TYPE drawType, uint32 vertex_index, uint32 vertex_count, uint32 texture_bufferID, vec4 color ) {
	RENDER_OBJECT_2D result = { drawType, vertex_index, vertex_count, texture_bufferID, color };
	return result;
}

internal RENDER_OBJECT_3D
RenderObject3D( MESH_DRAW_TYPE drawType, uint32 vertex_index, uint32 vertex_count, vec3 position, vec3 scale, quat orientation, uint32 texture_bufferID, vec4 color ) {
	RENDER_OBJECT_3D result = { drawType, vertex_index, vertex_count, position, scale, orientation, texture_bufferID, color };
	return result;
}

internal RENDER_TEXTURE
RenderTexture( uint32 bufferID, int32 width, int32 height ) {
	RENDER_TEXTURE result = { bufferID, width, height };
	return result;
}

internal RENDER_MESH
RenderMesh( MESH_DRAW_TYPE drawType, int32 vertex_count, VERTEX_TEXMESH_3D * vertex ) {
	RENDER_MESH result = { drawType, vertex_count, vertex };
	return result;
}

internal void
addRenderObject( RENDER_SYSTEM_3D * renderSystem, RENDER_OBJECT_3D renderObject ) {
	Assert( renderSystem->object_count < RENDER_SYSTEM__MAX_OBJECT_COUNT );
	renderSystem->object[ renderSystem->object_count++ ] = renderObject;
}

internal void
addRenderObject( RENDER_SYSTEM_3D * renderSystem, MESH_DRAW_TYPE drawType, uint32 vertex_index, uint32 vertex_count, vec3 position, vec3 scale, quat orientation, uint32 texture_bufferID, vec4 color ) {
	RENDER_OBJECT_3D renderObject = { drawType, vertex_index, vertex_count, position, scale, orientation, texture_bufferID, color };
	addRenderObject( renderSystem, renderObject );
}

//----------
// vertex buffer functions
//----------

internal int32
getTempIndex( VERTEX_BUFFER * vertexBuffer ) {
	int32 result = vertexBuffer->temp_maxCount * vertexBuffer->temp_select + vertexBuffer->temp_count;
	return result;
}

internal int32
getPermIndex( VERTEX_BUFFER * vertexBuffer ) {
	int32 result = vertexBuffer->perm_baseIndex + vertexBuffer->perm_count;
	return result;
}

internal void
addTemp( VERTEX_BUFFER * vertexBuffer, VERTEX_TEXMESH_3D * vertex, int32 vertex_count ) {
	Assert( vertexBuffer->temp_count + vertex_count < vertexBuffer->temp_maxCount );
	// memcpy( vertexBuffer->temp_vertex + vertexBuffer->temp_count, vertex, sizeof( VERTEX_TEXMESH_3D ) * vertex_count );
	memcpy( vertexBuffer->buffer_base + ( vertexBuffer->temp_select * vertexBuffer->temp_maxCount + vertexBuffer->temp_count ), vertex, sizeof( VERTEX_TEXMESH_3D ) * vertex_count );
	vertexBuffer->temp_count += vertex_count;
}

internal void
addVertex( VERTEX_BUFFER * vertexBuffer, VERTEX_TEXMESH_3D vertex ) {
	addTemp( vertexBuffer, &vertex, 1 );
}

internal void
addVertex( VERTEX_BUFFER * vertexBuffer, vec3 position, vec2 texCoord ) {
	VERTEX_TEXMESH_3D vertex = VertexTexMesh3D( position, texCoord );
	addTemp( vertexBuffer, &vertex, 1 );
}

internal void
addPerm( VERTEX_BUFFER * vertexBuffer, VERTEX_TEXMESH_3D * vertex, int32 vertex_count ) {
	Assert( ( vertexBuffer->perm_count + vertex_count ) < vertexBuffer->perm_maxCount );
	memcpy( vertexBuffer->buffer_base + ( vertexBuffer->perm_baseIndex + vertexBuffer->perm_count ), vertex, sizeof( VERTEX_TEXMESH_3D ) * vertex_count );
	vertexBuffer->perm_count += vertex_count;
}

internal void
beginTask( RENDER_SYSTEM_2D * renderSystem, MESH_DRAW_TYPE drawType, uint32 texture_bufferID, vec4 modColor ) {
	VERTEX_BUFFER * vertexBuffer = renderSystem->vertexBuffer;
	
	Assert( !renderSystem->taskIsActive );
	Assert( renderSystem->object_count < RENDER_SYSTEM__MAX_OBJECT_COUNT );
	
	renderSystem->taskIsActive = true;
	
	RENDER_OBJECT_2D tempObject = {};
	tempObject.drawType         = drawType;
	tempObject.vertex_index     = getTempIndex( vertexBuffer );
	tempObject.texture_bufferID = texture_bufferID;
	tempObject.color            = modColor;
	renderSystem->tempObject = tempObject;
}

internal void
addQuad( RENDER_SYSTEM_2D * renderSystem, rect vert_bound, rect tex_bound ) {
	VERTEX_BUFFER * vertexBuffer = renderSystem->vertexBuffer;
	
	Assert( renderSystem->taskIsActive );
	
	VERTEX_TEXMESH_3D vertexA = VertexTexMesh3D( vert_bound.left,  vert_bound.bottom, 0.0f,    tex_bound.left,  tex_bound.bottom );
	VERTEX_TEXMESH_3D vertexB = VertexTexMesh3D( vert_bound.right, vert_bound.bottom, 0.0f,    tex_bound.right, tex_bound.bottom );
	VERTEX_TEXMESH_3D vertexC = VertexTexMesh3D( vert_bound.right, vert_bound.top,    0.0f,    tex_bound.right, tex_bound.top );
	VERTEX_TEXMESH_3D vertexD = VertexTexMesh3D( vert_bound.left,  vert_bound.top,    0.0f,    tex_bound.left,  tex_bound.top );
	
	switch( renderSystem->tempObject.drawType ) {
		case meshDrawType_triangles: {
			addVertex( vertexBuffer, vertexA );
			addVertex( vertexBuffer, vertexB );
			addVertex( vertexBuffer, vertexC );
			
			addVertex( vertexBuffer, vertexA );
			addVertex( vertexBuffer, vertexC );
			addVertex( vertexBuffer, vertexD );
			
			renderSystem->tempObject.vertex_count += 6;
		} break;
		
		case meshDrawType_lineLoop:
		case meshDrawType_triangleFan: {
			addVertex( vertexBuffer, vertexA );
			addVertex( vertexBuffer, vertexB );
			addVertex( vertexBuffer, vertexC );
			addVertex( vertexBuffer, vertexD );
			
			renderSystem->tempObject.vertex_count += 4;
		} break;
		
		default: {
			InvalidCodePath;
		};
	}
}

internal void
endTask( RENDER_SYSTEM_2D * renderSystem ) {
	Assert( renderSystem->taskIsActive );
	
	renderSystem->taskIsActive = false;
	if( renderSystem->tempObject.vertex_count > 0 ) {
		renderSystem->object[ renderSystem->object_count++ ] = renderSystem->tempObject;
	}
}

internal void
beginTask( RENDER_SYSTEM_3D * renderSystem, MESH_DRAW_TYPE drawType, uint32 texture_bufferID, vec4 modColor ) {
	VERTEX_BUFFER * vertexBuffer = renderSystem->vertexBuffer;
	
	Assert( !renderSystem->taskIsActive );
	Assert( renderSystem->object_count < RENDER_SYSTEM__MAX_OBJECT_COUNT );
	
	renderSystem->taskIsActive = true;
	
	RENDER_OBJECT_3D tempObject = {};
	tempObject.drawType         = drawType;
	tempObject.vertex_index     = getTempIndex( vertexBuffer );
	tempObject.texture_bufferID = texture_bufferID;
	tempObject.color            = modColor;
	tempObject.scale            = Vec3( 1.0f, 1.0f, 1.0f );
	tempObject.orientation      = QuatDef();
	renderSystem->tempObject = tempObject;
}

internal void
addQuad( RENDER_SYSTEM_3D * renderSystem, vec3 * pos, vec2 * tex ) {
	VERTEX_BUFFER * vertexBuffer = renderSystem->vertexBuffer;
	
	Assert( renderSystem->taskIsActive );
	
	switch( renderSystem->tempObject.drawType ) {
		case meshDrawType_triangles: {
			addVertex( vertexBuffer, pos[ 0 ], tex[ 0 ] );
			addVertex( vertexBuffer, pos[ 1 ], tex[ 1 ] );
			addVertex( vertexBuffer, pos[ 2 ], tex[ 2 ] );
			
			addVertex( vertexBuffer, pos[ 0 ], tex[ 0 ] );
			addVertex( vertexBuffer, pos[ 2 ], tex[ 2 ] );
			addVertex( vertexBuffer, pos[ 3 ], tex[ 3 ] );
			
			renderSystem->tempObject.vertex_count += 6;
		} break;
		
		case meshDrawType_lineLoop:
		case meshDrawType_triangleFan: {
			addVertex( vertexBuffer, pos[ 0 ], tex[ 0 ] );
			addVertex( vertexBuffer, pos[ 1 ], tex[ 1 ] );
			addVertex( vertexBuffer, pos[ 2 ], tex[ 2 ] );
			addVertex( vertexBuffer, pos[ 3 ], tex[ 3 ] );
			
			renderSystem->tempObject.vertex_count += 4;
		} break;
		
		default: {
			InvalidCodePath;
		};
	}
}

internal void
endTask( RENDER_SYSTEM_3D * renderSystem ) {
	Assert( renderSystem->taskIsActive );
	
	renderSystem->taskIsActive = false;
	if( renderSystem->tempObject.vertex_count > 0 ) {
		renderSystem->object[ renderSystem->object_count++ ] = renderSystem->tempObject;
	}
}

#endif 	// STD_INCLUDE_FUNC